# Springbot Magento1 Extension

[![Build Status](https://circleci.com/gh/springbot/magento1-plugin/tree/master.svg?style=shield&circle-token=917f2a6a984676582107147e552c8d83cf85849f)](https://circleci.com/gh/springbot/magento1-plugin/tree/master)
[![GPL licensed](https://img.shields.io/badge/license-GPL-blue.svg)](https://github.com/springbot/magento1-plugin/blob/master/LICENSE.md)

### Springbot Integration for Magento1
Springbot, an eCommerce marketing platform designed for small to mid-sized online businesses, helps eCommerce stores 
grow revenue by taking smarter, data-driven marketing actions. If you would like more information on how Springbot can
help grow your business we encourage you to [schedule a free demo](http://go.springbot.com/l/61912/2016-10-04/k1xhkr).

The Magento1 Springbot extension allows Magento 1.x stores integrate with the Springbot platform. [If you're on 
Magento2, you can find our Magento2 extension here](https://github.com/springbot/magento2-plugin). 

### Requirements
* Magento CE >= 1.4
* Magento EE >= 1.8
* PHP >= 5.3

### Installation
Before installing Springbot (or any Magento extension), *ensure that Magento compilation is disabled*. This setting can 
be found in your Magento admin under System > Tools > Compilation. Once disabled, you can begin installation. After 
installation is complete you can run the compilation process so that all the newly copied files get included in your
compiled source files. If compilation was turned on before, be absolutely sure you have re-run the compilation process 
before re-enabling it. If compilation was disabled before starting then you do not need to change anything.

To install the extension, copy the `app` and `skin` directories directly into your Magento root directory. The Springbot
extension files should automatically merge into your Magento directory.

If you are you upgrading from a previous version (either our 1.x or an earlier 2.x version) simply copy the files over
the existing Springbot files. Your file utility may prompt you to overwrite existing files, in which case you should 
select yes. It is not necessary to delete older files from previous versions and they will not affect the speed or
performance of your store, but some administrators may choose to do so for version control purposes.

Once installed, be sure to clear your Magento cache so that the layout and configuration changes take effect. Your cache
management can be found in your Magento admin under System > Configuration > Cache Management.

### About the Integration

We've designed our integration to be as lightweight as possible. The extension exposes several Springbot specific
endpoints to keep your store synced with our service. Once synced, all marketing actions are performed within the 
Springbot dashboard located at https://app.springbot.com.

Springbot utilizes a job queuing system to defer sync jobs so that they may be run asynchronously. By doing this we are
able to avoid executing sync related tasks on page loads. Instead, a special queue endpoint is exposed via the included
API that our sync service requests on a periodic basis. When first installed, the sync service performs a full 
retroactive sync of all existing store data. From then on, when an action is performed on your store that results in the
creation/modification/deletion of data, a lightweight job is added to the queue to be processed asynchronously at a 
later time.

The extension also automatically places a small asynchronous javascript snippet in the footer of each page to track 
visitors and load 3rd party integrations such as Adroll. Because it is loaded in the background after the page has fully 
rendered, the end result is zero impact on the frontend rendering of your pages.

### Testing
To run unit tests locally, ensure phpunit is installed and run the following command:

```bash
phpunit --bootstrap=Test/Springbot/bootstrap.php Test/Springbot/Unit
```

### Committing and Contributing
If you are contributing to this project please create a branch with the issue number as the branch name. Ensure any new 
functionality includes proper testing. When creating new master releases please tag your release accordingly.

```bash
git add -A
git commit -m "My Changes"
git tag 2.x.x.x
git push origin --tags
```