<?php

class Springbot_Main_Block_Product extends Mage_Core_Block_Template
{
    private $product;
    private $store;

    protected function _construct()
    {
        $this->product = Mage::registry('current_product');
        $this->store = Mage::app()->getStore();
    }

    /**
     * Return the SKU for the product on the current page
     */
    public function getSku()
    {
        if ($this->product) {
            return urlencode($this->product->getSku());
        }
        return null;
    }

    /**
     * @return string|null
     */
    public function getStoreGuid()
    {
        $storeId = $this->getStoreId();
        $guid = Mage::getStoreConfig("springbot/main/store_guid_{$storeId}");
        $guid = strtolower($guid);
        $guid = str_replace('-', '', $guid);
        return $guid;
    }

    /**
     * @return string
     */
    public function getCurrentUrl()
    {
        return urlencode(Mage::helper('core/url')->getCurrentUrl());
    }

    /**
     * @return string
     */
    public function getApiUrl()
    {
        return Mage::getStoreConfig("springbot/main/api_url");
    }

    /**
     * Return the entity ID for the product on the current page
     */
    public function getId()
    {
        if ($this->product) {
            return $this->product->getId();
        }
        return null;
    }

    /**
     * Return the current store ID
     */
    public function getStoreId()
    {
        if ($this->store) {
            return $this->store->getStoreId();
        }
        return null;
    }

}