<?php

abstract class Springbot_Main_Model_Entity_AbstractEntity extends Mage_Core_Model_Abstract
{
    protected $collection;
    protected $storeId;

    const path = null;
    const singular = null;
    const plural = null;
    const page_size = 50;

    /**
     * @param $entity
     * @return array
     */
    abstract public function parse($entity);

    /**
     * @param $collection
     * @param $storeId
     * @return mixed
     */
    abstract protected function filterCollectionByStore($collection, $storeId);

    /**
     * Springbot_Main_Model_Entity_AbstractEntity constructor.
     * @param $params
     */
    public function __construct($params)
    {
        $this->storeId = $params['store_id'];
    }

    /**
     * @return mixed
     */
    protected function getModel()
    {
        return Mage::getModel(static::path);
    }

    /**
     * @param int $storeId
     * @param int $page
     * @return mixed
     */
    public function getFilteredCollection($storeId, $page = 1)
    {
        if (!is_numeric($page) || ($page < 1)) {
            $page = 1;
        }
        // Some of the model's setStore() method like Sales Quote require an instance of the core store model, but since this is rare and causes error when it isn't an object of that class, lets just filter after the fact
        $collection = $this
            ->getModel(static::path)
            ->getCollection();

        // If we filter the collections at the end, why do we need to filter by store when we retirieve them?
        $collection = $this->filterCollectionByStore($collection, $storeId);

        $page = max(1, (int)$page);
        $start = (($page - 1) * self::page_size);
        $collection
            ->getSelect()
            ->limit(self::page_size, $start);
        return $collection;

    }

    /**
     * @param int $storeId
     * @return mixed
     */
    public function getCount($storeId)
    {
        $collection = $this->getFilteredCollection($storeId);
        $collection->getSelect()->limit(null);
        return $collection->getSize();
    }

    /**
     * @param array $data
     * @throws Exception
     * @return array
     */
    public function create($storeId, $data)
    {
        throw new Exception('create() not implemented for entity');
    }

    /**
     * @param $storeId
     * @param $id
     * @return array
     * @throws Exception
     */
    public function one($storeId, $id)
    {
        $entity = $this->getModel()->setStoreId($storeId)->load($id);
        if (!$entity->getId()) {
            throw new Exception('Entity not found', 404);
        }
        return $this->parse($entity);
    }

    /**
     * @param $storeId
     * @param $page
     * @return array
     */
    public function multiple($storeId, $page)
    {
        return $this->parseCollection($storeId, $this->getFilteredCollection($storeId, $page));
    }

    /**
     * @param int $storeId
     * @param $collection
     * @return array
     */
    protected function parseCollection($storeId, $collection)
    {
        $ret = array();
        foreach ($collection as $entity) {
            if ($storeId) {
                $entity->setStoreId($storeId);
            }
            $entity->load($entity->getId());
            $ret[] = $this->parse($entity);
        }
        return $ret;
    }

    /**
     * @param $address
     * @return array|null
     */
    protected function parseAddress($address)
    {
        if (!is_object($address)) {
            return null;
        }
        return array(
            'created_at' => $address->getCreatedAt(),
            'updated_at' => $address->getUpdatedAt(),
            'is_active'  => $address->getIsActive(),
            'firstname'  => $address->getFirstname(),
            'lastname'   => $address->getLastname(),
            'city'       => $address->getCity(),
            'region'     => $address->getRegion(),
            'postcode'   => $address->getPostcode(),
            'country_id' => $address->getCountryId(),
            'telephone'  => $address->getTelephone(),
            'street'     => trim(implode("\n", $address->getStreet())),
        );
    }

}