<?php

class Springbot_Main_Model_Entity_AttributeSet extends Springbot_Main_Model_Entity_AbstractEntity
{
    const path = 'eav/entity_attribute_set';
    const singular = 'attribute_set';
    const plural = 'attribute_sets';

    private $mappings = array();
    private $types = array(
        'catalog_product' => 'product',
        'customer'        => 'customer'
    );

    /**
     * Springbot_Main_Model_Entity_AttributeSet constructor.
     *
     * In addition to the normal steps, for attribute sets we need to load the various entity types and their
     * corresponding entity_type_id.
     *
     * @param array $params
     */
    public function __construct($params)
    {
        parent::__construct($params);
        foreach ($this->types as $model => $name) {
            $id = Mage::getSingleton('eav/config')
                ->getEntityType($model)
                ->getId();
            $this->mappings[$id] = $name;
        }
    }

    /**
     * Limit to only product and customer sets
     *
     * @param int $storeId
     * @param null $page
     * @return mixed
     */
    public function getFilteredCollection($storeId, $page = null)
    {
        return parent::getFilteredCollection($storeId, $page)
            ->addFieldToFilter('entity_type_id', array('in', array_keys($this->mappings)));
    }

    /**
     * Attribute sets are unique in that we need to manually retrieve the attributes depending on which type of set
     *
     * @param $entity
     * @return array
     */
    public function parse($entity)
    {
        $type = $this->mappings[$entity->getEntityTypeId()];
        $attributes = array();
        if (in_array($type, array('customer', 'product'))) {
            $attributesCollection = Mage::getResourceModel('eav/entity_attribute_collection')
                ->setAttributeSetFilter($entity->getId())
                ->addFieldToFilter('is_user_defined', array('gt' => 0))
                ->getItems();
            foreach ($attributesCollection as $attribute) {
                $options = array();
                if ($attribute->usesSource()) {
                    try {
                        $optionsArray = $attribute->getSource()->getAllOptions();
                    } catch (Exception $e) {
                        $optionsArray = array();
                    }
                    foreach ($optionsArray as $option) {
                        $options[] = $option['value'];
                    }
                }
                $attributes[] = array(
                    'attribute_id' => $attribute->getId(),
                    'code'         => $attribute->getAttributeCode(),
                    'type'         => $attribute->getBackendType(),
                    'required'     => !!$attribute->getIsRequired(),
                    'scope'        => 'global',
                    'options'      => $options
                );
            }
        }

        return array(
            'attribute_set_id' => $entity->getId(),
            'name'             => $entity->getAttributeSetName(),
            'type'             => $type,
            'attributes'       => $attributes
        );
    }

    /**
     * @param $collection
     * @param int $storeId
     * @return mixed
     */
    protected function filterCollectionByStore($collection, $storeId)
    {
        return $collection;
    }

}