<?php

class Springbot_Main_Model_Entity_Cart extends Springbot_Main_Model_Entity_AbstractEntity
{
    const path = 'sales/quote';
    const singular = 'cart';
    const plural = 'carts';

    /**
     * @param $entity
     * @return array
     */
    public function parse($entity)
    {
        return array(
            'cart_id'             => $entity->getId(),
            'converted_at'        => $entity->getConvertedAt(),
            'customer_id'         => $entity->getCustomerId(),
            'customer_prefix'     => $entity->getCustomerPrefix(),
            'customer_firstname'  => $entity->getCustomerFirstname(),
            'customer_lastname'   => $entity->getCustomerLastname(),
            'customer_middlename' => $entity->getCustomerMiddlename(),
            'customer_suffix'     => $entity->getCustomerSuffix(),
            'checkout_method'     => $entity->getCheckoutMethod(),
            'remote_ip'           => $entity->getRemoteIp(),
            'cart_user_agent'     => $entity->getCartUserAgent(),
            'customer_is_guest'   => $entity->getCustomerIsGuest(),
            'created_at'          => $entity->getCreatedAt(),
            'updated_at'          => $entity->getUpdatedAt(),
            'customer_email'      => $entity->getCustomerEmail(),
            'items'               => $this->parseItems($entity)
        );
    }

    /**
     * @param $collection
     * @param int $storeId
     * @return mixed
     */
    protected function filterCollectionByStore($collection, $storeId)
    {
        return $collection->addFieldToFilter('store_id', $storeId);
    }

    /**
     * Parses the individual line items on a cart (aka quote)
     *
     * @param $entity
     * @return array
     */
    private function parseItems($entity)
    {
        $items = array();
        foreach ($entity->getAllVisibleItems() as $item) {
            $productId = null;
            $sku = null;
            $parentProductId = null;
            $parentSku = null;
            $imageUrl = null;
            if ($children = $item->getChildren()) {
                foreach ($children as $child) {
                    $parentProduct = Mage::getModel('catalog/product')->load($item->getProductId());
                    $productId = $child->getProductId();
                    $sku = $child->getSku();
                    $parentProductId = $parentProduct->getId();
                    $productType = $child->getProductType();
                    $parentSku = $parentProduct->getSku();
                    $imageUrl = $parentProduct->getImageUrl();
                    break;
                }
            } else {
                $parentProduct = Mage::getModel('catalog/product')->load($item->getProductId());
                $productId = $item->getId();
                $sku = $item->getSku();
                $productType = $item->getProductType();
                $imageUrl = $parentProduct->getImageUrl();
            }

            $product = $item->getProduct();
            $items[] = array(
                'name'              => $item->getName(),
                'qty'               => $item->getQty(),
                'product_id'        => $productId,
                'sku'               => $sku,
                'parent_product_id' => $parentProductId,
                'parent_sku'        => $parentSku,
                'product_type'      => $productType,
                'landing_url'       => $product->getProductUrl(false),
                'image_url'         => $imageUrl,
            );
        }
        return $items;
    }

}