<?php

class Springbot_Main_Model_Entity_Customer extends Springbot_Main_Model_Entity_AbstractEntity
{
    const path = 'customer/customer';
    const singular = 'customer';
    const plural = 'customers';

    private $ignore = array('password_hash', 'default_billing', 'default_shipping', 'email', 'firstname', 'lastname');

    /**
     * @param $entity
     *
     * @return array
     */
    public function parse($entity)
    {
        return array(
            'customer_id'         => $entity->getId(),
            'first_name'          => $entity->getFirstname(),
            'last_name'           => $entity->getLastname(),
            'email'               => $entity->getEmail(),
            'attribute_set_id'    => $entity->getAttributeSetId(),
            'has_purchase'        => $entity->hasPurchase(),
            'customer_attributes' => $this->parseCustomAttributes($entity),
            'billing_address'     => $this->parseAddress($entity->getDefaultBillingAddress()),
            'shipping_address'    => $this->parseAddress($entity->getDefaultShippingAddress()),
        );
    }

    /**
     * @param $collection
     * @param int $storeId
     *
     * @return mixed
     */
    protected function filterCollectionByStore($collection, $storeId)
    {
        return $collection->addFieldToFilter('store_id', $storeId);
    }

    /**
     * @param $entity
     *
     * @return array
     */
    private function parseCustomAttributes($entity)
    {
        $return     = array();
        $helper     = Mage::helper('springbot_main/attributes');
        $model      = $entity;
        $attributes = $helper->getAttributesBySet($model->getAttributeSetId());
        foreach ($attributes as $attribute) {
            $code = $attribute->getAttributeCode();
            $name = $attribute->getName();
            if (!in_array($code, $this->ignore) && $model->hasData($code)) {
                if ($attribute->usesSource()) {
                    $value = $helper->getOptionText($attribute, $model->getData($code));
                } else {
                    $value = $model->getData($code);
                }
                $return[] = array(
                    'name'  => $name,
                    'value' => $value
                );
            }
        }

        return $return;
    }

}
