<?php

class Springbot_Main_Model_Entity_Inventory extends Springbot_Main_Model_Entity_AbstractEntity
{
    const path = 'cataloginventory/stock_item';
    const singular = 'inventory';
    const plural = 'inventories';

    /**
     * @param $entity
     * @return array
     */
    public function parse($entity)
    {
        $product = Mage::getModel('catalog/product')->load($entity->getProductId());
        $parentIds = $product->getParentIdsByChild($entity->getProductId());

        if ($product->getTypeId() == "simple") {
            $parentIds = Mage::getModel('catalog/product_type_grouped')
                ->getParentIdsByChild($entity->getProductId());
            if (!$parentIds) {
                $parentIds = Mage::getModel('catalog/product_type_configurable')
                    ->getParentIdsByChild($entity->getProductId());
            }
        }
        if (!$parentIds) {
            $parentIds = array();
        }
        $parentSkus = array();
        foreach ($parentIds as $parentId) {
            $parentSkus[] = Mage::getModel('catalog/product')
                ->load($parentId)
                ->getSku();
        }

        return array(
            'entity_id'        => $entity->getId(),
            'product_id'       => $entity->getProductId(),
            'system_managed'   => !!$entity->getManageStock(),
            'out_of_stock_qty' => $entity->getMinQty(),
            'quantity'         => $entity->getQty(),
            'item_id'          => $entity->getItemId(),
            'is_in_stock'      => !!$entity->getIsInStock(),
            'min_sale_qty'     => $entity->getMinSaleQty(),
            'parent_ids'       => $parentIds,
            'parent_skus'      => $parentSkus,
            'sku_fulfillment'  => $product->getSku()
        );
    }

    /**
     * @param $collection
     * @param int $storeId
     * @return mixed
     */
    protected function filterCollectionByStore($collection, $storeId)
    {
        return $collection;
    }

}