<?php

class Springbot_Main_Model_Entity_Order extends Springbot_Main_Model_Entity_AbstractEntity
{
    const path = 'sales/order';
    const singular = 'order';
    const plural = 'orders';

    private $redirectIds = null;
    private $filteredAttributes = array(
        'attribute_set_id',
        'custom_design',
        'custom_design_from',
        'custom_design_to',
        'custom_layout_update',
        'description',
        'entity_id',
        'entity_type_id',
        'gift_message_available',
        'group_price_changed',
        'has_options',
        'image',
        'image_label',
        'is_in_stock',
        'is_recurring',
        'is_salable',
        'meta_description',
        'meta_keyword',
        'meta_title',
        'msrp_display_actual_price_type',
        'msrp_enabled',
        'news_from_date',
        'news_to_date',
        'options_container',
        'page_layout',
        'quantity_and_stock_status',
        'request_path',
        'required_options',
        'short_description',
        'sku',
        'small_image',
        'small_image_label',
        'special_from_date',
        'special_to_date',
        'stock_item',
        'tax_class_id',
        'tax_class_id',
        'thumbnail',
        'thumbnail_label',
        'tier_price_changed',
        'type_id',
        'url',
        'url_key',
        'url_path',
    );

    /**
     * @param $entity
     * @return array
     */
    public function parse($entity)
    {
        $this->redirectIds = null;
        $quoteId = $entity->getQuoteId();
        $orderId = $entity->getId();
        return array(
            'order_id'            => $orderId,
            'quote_id'            => $quoteId,
            'increment_id'        => $entity->getIncrementId(),
            'customer_email'      => $entity->getCustomerEmail(),
            'redirect_mongo_id'   => $this->getRedirectId($entity),
            'redirect_mongo_ids'  => $this->getRedirectIds($entity),
            'customer_id'         => $entity->getCustomerId(),
            'grand_total'         => $entity->getGrandTotal(),
            'remote_ip'           => $entity->getRemoteIp(),
            'status'              => $entity->getStatus(),
            'state'               => $entity->getState(),
            'customer_is_guest'   => !!$entity->getCustomerIsGuest(),
            'created_at'          => $entity->getCreatedAt(),
            'discount_amount'     => $entity->getDiscountAmount(),
            'total_paid'          => $entity->getTotalPaid(),
            'shipping_method'     => $entity->getShippingMethod(),
            'shipping_amount'     => $entity->getShippingAmount(),
            'payment'             => 'online',
            'coupon_code'         => $entity->getCouponCode(),
            'order_currency_code' => $entity->getOrderCurrencyCode(),
            'base_tax_amount'     => $entity->getBaseTaxAmount(),
            'cart_user_agent'     => Mage::getModel('springbot_main/trackable_quote')->get($quoteId, 'user_agent'),
            'order_user_agent'    => Mage::getModel('springbot_main/trackable_order')->get($orderId, 'user_agent'),
            'shipments'           => $this->parseShipments($entity),
            'items'               => $this->parseItems($entity),
        );
    }

    /**
     * @param $collection
     * @param int $storeId
     * @return mixed
     */
    protected function filterCollectionByStore($collection, $storeId)
    {
        return $collection->addFieldToFilter('store_id', $storeId);
    }

    /**
     * @param $storeId
     * @param array $data
     * @return array
     */
    public function create($storeId, $data)
    {
        $orderBuilder = Mage::getModel('springbot_main/entity_order_orderBuilder', $data);
        return $this->parse($orderBuilder->buildOrder());
    }

    /**
     * @param $order
     * @return array
     */
    private function parseShipments($order)
    {
        $ret = array();
        foreach ($order->getShipmentsCollection() as $shipment) {
            $trackingNumber = null;
            $carrierCode = null;
            foreach ($shipment->getAllTracks() as $track) {
                $trackingNumber = $track->getNumber();
                $carrierCode = $track->getCarrierCode();
            }
            $items = array();
            foreach ($shipment->getAllItems() as $item) {
                $items[] = array(
                    'sku'        => $item->getSku(),
                    'name'       => $item->getName(),
                    'product_id' => $item->getProductId(),
                    'qty'        => $item->getQty()
                );
            }
            $title = null;
            if (isset($track) && is_object($track)) {
                $title = $track->getTitle();
            }
            $ret[] = array(
                'shipment_id'     => $shipment->getId(),
                'tracking_number' => $trackingNumber,
                'carrier_code'    => $carrierCode,
                'title'           => $title,
                'ship_to'         => $order->getCustomerName(),
                'shipment_status' => $shipment->getShipmentStatus(),
                'items'           => $items
            );
        }
        return $ret;
    }

    /**
     * @param $order
     * @return array
     */
    private function parseItems($order)
    {
        $parentSku = null;
        $tempItems = array();
        foreach ($order->getAllItems() as $item) {
            $product = Mage::getModel('catalog/product')->load($item->getProductId());
            $parentId = null;
            if (($parent = $item->getParentItem()) && ($parentProduct = $parent->getProduct())) {
                $parentId = $parentProduct->getId();
            }

            $itemArray = array(
                'sku'                => $product->getSku(),
                'sku_fulfillment'    => $item->getSku(),
                'parent_product_id'  => $parentId,
                'qty_ordered'        => $item->getQtyOrdered(),
                'name'               => $item->getName(),
                'sell_price'         => $item->getBasePrice(),
                'wgt'                => $item->getWeight(),
                'landing_url'        => $product->getProductUrl(),
                'image_url'          => $product->getImageUrl(),
                'desc'               => $product->getDescription(),
                'product_id'         => $item->getId(),
                'product_type'       => $product->getTypeId(),
                'category_ids'       => $product->getCategoryIds(),
                'all_category_ids'   => $product->getCategoryIds(),
                'attribute_set_id'   => $product->getAttributeSetId(),
                'product_attributes' => $this->filterAttributes($product->getData()),
            );

            if (isset($tempItems[$item->getSku()])) {
                $tempItems[$item->getSku()] = $this->mergeItem($tempItems[$item->getSku()], $itemArray);
            }
            else {
                $tempItems[$item->getSku()] = $itemArray;
            }
        }
        foreach ($tempItems as &$tempItem) {
            $tempItem['product_attributes'] = $this->expandAttributes($tempItem['product_attributes']);
        }
        return array_values($tempItems);
    }

    private function filterAttributes($attributes)
    {
        $ret = array();
        foreach ($attributes as $key => $value) {
            if (!is_array($value) && !in_array($key, $this->filteredAttributes)) {
                $ret[$key] = $value;
            }
        }
        return $ret;
    }

    private function expandAttributes($attributes)
    {
        $ret = array();
        foreach ($attributes as $key => $value) {
            if (!is_array($value) && !in_array($key, $this->filteredAttributes)) {
                $ret[] = array(
                    'name'  => $key,
                    'value' => $value
                );
            }
        }
        return $ret;
    }

    /**
     * @param $item1
     * @param $item2
     * @return mixed
     */
    private function mergeItem($item1, $item2)
    {
        foreach ($item2 as $key => $value) {
            $item1[$key] = $this->coalesce($key, $item1, $item2);
        }
        return $item1;
    }

    /**
     * @param string $key
     * @param $value1
     * @param $value2
     * @return mixed
     */
    private function coalesce($key, $value1, $value2)
    {
        if (isset($value1[$key]) && isset($value2[$key]) && is_array($value1[$key]) && is_array($value2[$key])) {
            return array_merge($value1[$key], $value2[$key]);
        }
        elseif (isset($value1[$key])) {
            return $value1[$key];
        }
        elseif (isset($value2[$key])) {
            return $value2[$key];
        }
        return null;
    }

    /**
     * @param $order
     * @return array
     */
    private function getRedirectIds($order)
    {
        if ($this->redirectIds !== null) {
            return $this->redirectIds;
        }

        $redirectCollection = Mage::getModel('springbot_main/redirect_quote')
            ->getCollection()
            ->setOrder('id', 'asc');

        if ($order->getQuoteId()) {
            $redirectCollection->addFieldToFilter(
                array('quote_id', 'order_id'),
                array(
                    array('eq' => $order->getQuoteId()),
                    array('eq' => $order->getId())
                )
            );
        }
        else {
            $redirectCollection->addFieldToFilter(
                'order_id',
                array('eq' => $order->getId())
            );
        }

        $redirects = array();
        foreach ($redirectCollection as $redirect) {
            $redirects[] = $redirect->getRedirectId();
        }
        $this->redirectIds = $redirects;
        return $redirects;
    }

    /**
     * @param $order
     * @return string|null
     */
    private function getRedirectId($order)
    {
        $redirectIds = $this->getRedirectIds($order);
        if ($redirectIds) {
            return end($redirectIds);
        }
        return null;
    }

}