<?php

class Springbot_Main_Model_Entity_Order_CustomerBuilder
{
    private $data;
    private $helper;

    /**
     * Springbot_Main_Model_Entity_Order_CustomerBuilder constructor.
     * @param $data
     */
    public function __construct($data)
    {
        $this->data = $data;
        $this->helper = Mage::helper('springbot_main');
    }

    /**
     * @return bool
     */
    public function createCustomer()
    {
        $customer = $this->getExistingCustomer($this->helper->fetch($this->data, array('buyer_email')));
        if (!$customer) {
            $password = Mage::helper('core')->getRandomString(6);
            $customer = Mage::getModel('customer/customer')
                ->setData('firstname', $this->getFirstname())
                ->setData('lastname', $this->getLastname())
                ->setData('website_id', 0)
                ->setData('group_id', 0)
                ->setData('email', $this->helper->fetch($this->data, array('buyer_email')))
                ->setData('confirmation', $password)
                ->setPassword($password)
                ->save();
        }
        $region = $this->getRegion();
        $customerAddress = Mage::getModel('customer/address')
            ->setFirstname($this->getFirstname())
            ->setLastname($this->getLastname())
            ->setCompany($this->getCompany())
            ->setRegion($region)
            ->setRegionId($this->getRegionId($region))
            ->setCountryId($this->helper->fetch($this->data, array('shipping_address', 'CountryCode')))
            ->setCity($this->helper->fetch($this->data, array('shipping_address', 'City')))
            ->setPostcode($this->helper->fetch($this->data, array('shipping_address', 'PostalCode')))
            ->setPhone($this->helper->fetch($this->data, array('shipping_address', 'Phone')))
            ->setStreet($this->getStreet())
            ->setCustomer($customer)
            ->setIsDefaultBilling(true)
            ->setIsDefaultShipping(true)
            ->save();
        $customer->setDefaultBilling($customerAddress->getEntityId())
            ->setDefaultShipping($customerAddress->getEntityId())
            ->save();
        Mage::log($customer->debug(), Zend_Log::DEBUG, 'sb.log');
        return $customer;
    }

    /**
     * @return mixed
     */
    private function getCompany()
    {
        $buyerName = $this->helper->fetch($this->data, array('buyer_name'));
        $shippingAddressName = $this->helper->fetch($this->data, array('shipping_address', 'Name'));
        if ($buyerName != $shippingAddressName) {
            return $shippingAddressName;
        }
    }

    /**
     * @return mixed
     */
    private function getFirstname()
    {
        return preg_replace('/\s.*$/', '', $this->helper->fetch($this->data, array('buyer_name')));
    }

    /**
     * @return mixed
     */
    private function getLastname()
    {
        return str_replace($this->getFirstname(), '', $this->helper->fetch($this->data, array('buyer_name')));
    }

    /**
     * @return string
     */
    private function getStreet()
    {
        return $this->helper->fetch($this->data, array('shipping_address', 'AddressLine1'))
            . PHP_EOL . $this->helper->fetch($this->data, array('shipping_address', 'AddressLine2'))
            . PHP_EOL . $this->helper->fetch($this->data, array('shipping_address', 'AddressLine3'));
    }

    /**
     * @return mixed
     */
    private function getRegion()
    {
        $region = Mage::getModel('directory/region')->loadByName(
            $this->helper->fetch($this->data, array('shipping_address', 'StateOrRegion')),
            $this->helper->fetch($this->data, array('shipping_address', 'CountryCode'))
        );

        // If we can't load by region, fail over and try to load by code
        if ($region->getId() == null) {
            $region = Mage::getModel('directory/region')->loadByCode(
                $this->helper->fetch($this->data, array('shipping_address', 'StateOrRegion')),
                $this->helper->fetch($this->data, array('shipping_address', 'CountryCode'))
            );
        }
        return $region;
    }

    /**
     * @param $region
     * @return null
     */
    private function getRegionId($region)
    {
        if ($region) {
            return $region->getId();
        }
        return null;
    }

    /**
     * @param $email
     * @return null
     */
    private function getExistingCustomer($email)
    {
        $customer = Mage::getModel('customer/customer')
            ->setWebsiteId(0)
            ->loadByEmail($email);
        if ($customer->getId()) {
            return $customer;
        }
        return null;
    }
}