<?php

class Springbot_Main_Model_Entity_Order_ItemBuilder
{

    private $data;
    private $helper;

    /**
     * Springbot_Main_Model_Entity_Order_Builder constructor.
     * @param $data
     */
    public function __construct($data)
    {
        $this->data = $data;
        $this->helper = Mage::helper('springbot_main');
    }

    /**
     * @param array $product
     * @return mixed
     * @throws Exception
     */
    public function makeOrderItem($product)
    {
        if (!is_object($product)) {
            throw new Exception("Invalid product as line item");
        }
        $qty = $this->helper->fetch($this->data, array('quantity_ordered'), null, true);
        if ($qty == 0) {
            throw new Exception("No quantity provided for product with sku: {$this->product()->getSku()}");
        }
        $this->setOptions($product);
        $options = $product->getTypeInstance(true)->getOrderOptions($product);
        $tax = $this->helper->fetch($this->data, array('item_tax', 'Amount'));
        $itemPrice = floatval($this->helper->fetch($this->data, array('item_price', 'Amount')));
        $price = $itemPrice / $qty;
        $rowTotal = $itemPrice + $tax;
        $orderItem = Mage::getModel('sales/order_item')
            ->setStoreId(0)
            ->setQuoteItemId(0)
            ->setQuoteParentItemId(null)
            ->setProductId($product->getId())
            ->setProductType($product->getTypeId())
            ->setQtyBackordered(null)
            ->setTotalQtyOrdered($qty)
            ->setQtyOrdered($qty)
            ->setName($product->getName())
            ->setSku($product->getSku())
            ->setPrice($price)
            ->setBasePrice($price)
            ->setTax($tax)
            ->setBaseTax($tax)
            ->setOriginalPrice($rowTotal)
            ->setRowTotal($rowTotal)
            ->setBaseRowTotal($rowTotal)
            ->setWeeeTaxApplied(array())
            ->setBaseWeeeTaxDisposition(0)
            ->setWeeeTaxDisposition(0)
            ->setBaseWeeeTaxRowDisposition(0)
            ->setWeeeTaxRowDisposition(0)
            ->setBaseWeeeTaxAppliedAmount(0)
            ->setBaseWeeeTaxAppliedRowAmount(0)
            ->setWeeeTaxAppliedAmount(0)
            ->setWeeeTaxAppliedRowAmount(0)
            ->setProductOptions($options);
        return $orderItem;
    }

    /**
     * @param $product
     * @return $this
     */
    private function setOptions($product)
    {
        $options = $product->getCustomOptions();
        $optionsByCode = array();
        foreach ($options as $option) {
            $quoteOption = Mage::getModel('sales/quote_item_option')
                ->setData($option->getData())
                ->setProduct($option->getProduct());
            $optionsByCode[$quoteOption->getCode()] = $quoteOption;
        }
        $product->setCustomOptions($optionsByCode);
        return $this;
    }

}