<?php

class Springbot_Main_Model_Entity_Product extends Springbot_Main_Model_Entity_AbstractEntity
{
    const path = 'catalog/product';
    const singular = 'product';
    const plural = 'products';

    /**
     * @param $entity
     * @return array
     */
    public function parse($entity)
    {
        $entity = $entity->load($entity->getId());
        $parentIds = $this->getParentIds($entity);
        return array(
            'product_id'              => $entity->getId(),
            'name'                    => $entity->getName(),
            'url_key'                 => $entity->getUrlKey(),
            'sku'                     => $entity->getSku(),
            'type'                    => $entity->getTypeId(),
            'status'                  => $entity->getStatus(),
            'visibility'              => $entity->getVisibility(),
            'msrp'                    => $entity->getMsrp(),
            'price'                   => $entity->getPrice(),
            'cost'                    => $entity->getCost(),
            'weight'                  => $entity->getWeight(),
            'created_at'              => $entity->getCreatedAt(),
            'updated_at'              => $entity->getUpdatedAt(),
            'description'             => $entity->getDescription(),
            'short_description'       => $entity->getShortDescription(),
            'category_ids'            => $entity->getCategoryIds(),
            'all_category_ids'        => $entity->getCategoryIds(),
            'special_price'           => $entity->getSpecialPrice(),
            'image_url'               => $this->getImageUrl($entity),
            'image_url_cached'        => $entity->getImageUrl(),
            'image_label'             => $entity->getImageLabel(),
            'custom_attribute_set_id' => $entity->getAttributeSetId(),
            'default_url'             => $entity->getProductUrl(),
            'url_in_store'            => $entity->getUrlInStore(),
            'url_id_path'             => $this->getUrlIdPath($entity),
            'parent_skus'             => $this->getParentSkus($parentIds),
            'parent_ids'              => $parentIds,
            'product_attributes'      => $this->getProductAttributes($entity)
        );
    }

    /**
     * @param $collection
     * @param int $storeId
     * @return mixed
     */
    protected function filterCollectionByStore($collection, $storeId)
    {
        if (Mage::getStoreConfig('springbot/main/category_filter') != false) {
            $rootCategoryId = Mage::getModel('core/store')->load($storeId)->getRootCategoryId();
            $rootCategory = Mage::getModel('catalog/category')->load($rootCategoryId);
            return $collection
            ->addStoreFilter($storeId)
            ->setStoreId($storeId)
            ->addCategoryFilter($rootCategory);
        } else {
            return $collection
            ->addStoreFilter($storeId)
            ->setStoreId($storeId);
        }
    }

    /**
     * @param $product
     * @return array
     */
    private function getParentIds($product)
    {
        $parentIds = array();
        $out = array();
        if ($product->getTypeId() == "simple" || $product->getTypeId() == "virtual") {
            $parentIds = Mage::getModel('catalog/product_type_grouped')->getParentIdsByChild($product->getId());
            if (!$parentIds) {
                $parentIds = Mage::getModel('catalog/product_type_configurable')
                    ->getParentIdsByChild($product->getId());
            }
            foreach ($parentIds as $parentId) {
                $product = Mage::getModel('catalog/product')->load($parentId);
                if ($product->getTypeId() != 'bundle') {
                    $out[] = $parentId;
                }
            }
        }
        return $out;
    }

    /**
     * @param array $parentIds
     * @return array
     */
    private function getParentSkus($parentIds)
    {
        $parentSkus = array();
        if (is_array($parentIds)) {
            foreach ($parentIds as $parentId) {
                $product = Mage::getModel('catalog/product')->load($parentId);
                if ($sku = $product->getSku()) {
                    $parentSkus[] = $sku;
                }
            }
        }
        return $parentSkus;
    }

    /**
     * @param $product
     * @return mixed
     */
    private function getUrlIdPath($product)
    {
        return Mage::getUrl(
            'catalog/product/view',
            array(
                'id'     => $product->getId(),
                '_store' => $product->getStoreId(),
            )
        );
    }

    /**
     * @param $product
     * @return null|string
     */
    private function getImageUrl($product)
    {
        if ($this->fileExists($image = $product->getImage())) {
            return $this->getMediaUrl($image);
        }
        else if ($this->fileExists($image = $product->getSmallImage())) {
            return $this->getMediaUrl($image);
        }
        else if ($this->fileExists($image = $product->getThumbnail())) {
            return $this->getMediaUrl($image);
        }
        else if ($product->getMediaGalleryImages() && $product->getMediaGalleryImages()->getSize() > 0) {
            // First item from gallery
            return $product->getMediaGalleryImages()->getFirstItem()->getUrl();
        }
        else {
            // if we get here, the image doesn't exist, return null
            return null;
        }
    }

    /**
     * @param string $image
     * @return string
     */
    private function getMediaUrl($image)
    {
        if (strpos($image, DIRECTORY_SEPARATOR) !== 0) {
            $image = DIRECTORY_SEPARATOR . $image;
        }
        return Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_MEDIA) . 'catalog/product' . $image;
    }

    /**
     * @param $file
     * @param $type
     * @return bool
     */
    private function fileExists($file, $type = Mage_Core_Model_Store::URL_TYPE_MEDIA)
    {
        $file = Mage::getBaseDir($type) . '/catalog/product/' . $file;
        return file_exists($file);
    }

    /**
     * @param $entity
     * @return array
     */
    private function getProductAttributes($entity)
    {
        return $entity->getData();
    }

}
