<?php

class Springbot_Main_Model_Migrate
{
    const record_limit = 250;

    private $resource;
    private $connection;

    public function __construct()
    {
        $this->resource = Mage::getSingleton('core/resource');
        $this->connection = $this->resource->getConnection('core_write');
    }

    /**
     * Migrate redirect data from old extension tables to new
     */
    public function migrateRedirects()
    {
        $tableName = $this->resource->getTableName('springbot_redirect');
        $query = "SELECT * FROM {$tableName} ORDER BY id LIMIT " . self::record_limit;
        $rows = $this->connection->fetchAll($query);
        $ids = array();
        $redirectModel = Mage::getModel('springbot_main/redirect_quote');
        foreach ($rows as $row) {
            $ids[] = $row['id'];
            $redirectModel->create($row['quote_id'], $row['redirect_id']);
        }
        if ($ids) {
            $this->deleteIds($tableName, $ids);
            if (count($ids) == self::record_limit) {
                Mage::getModel('springbot_queue/manager')->enqueue('Springbot_Main_Model_Migrate', 'migrateRedirects');
            }
        }
    }

    /**
     * Migrate trackables data from old extension tables to new
     */
    public function migrateTrackables()
    {
        $tableName = $this->resource->getTableName('springbot_trackable');
        $query = "SELECT * FROM {$tableName} ORDER BY id LIMIT " . self::record_limit;
        $rows = $this->connection->fetchAll($query);
        $ids = array();
        $trackableQuote = Mage::getModel('springbot_main/trackable_quote');
        $trackableOrder = Mage::getModel('springbot_main/trackable_order');
        foreach ($rows as $row) {
            if ($quoteId = $row['quote_id']) {
                $trackableQuote->create($quoteId, $row['type'], $row['value']);
            }
            if ($orderId = $row['order_id']) {
                $trackableOrder->create($orderId, $row['type'], $row['value']);
            }
            $ids[] = $row['id'];
        }
        if ($ids) {
            $this->deleteIds($tableName, $ids);
            if (count($ids) == self::record_limit) {
                Mage::getModel('springbot_queue/manager')->enqueue('Springbot_Main_Model_Migrate', 'migrateTrackables');
            }
        }
    }

    /**
     * @param string $tableName
     * @param array $ids
     * @return mixed
     */
    private function deleteIds($tableName, $ids)
    {
        $query = "DELETE FROM {$tableName} WHERE id IN (". implode(', ', $ids) .")";
        return $this->connection->exec($query);
    }

}
