<?php

class Springbot_Main_Model_Rewrite extends Mage_Core_Model_Abstract
{

    /**
     * @param $store
     * @param $idPath
     * @param $requestPath
     * @param $targetPath
     * @return bool
     */
    public function createRewrite($store, $idPath, $requestPath, $targetPath)
    {
        try {
            if ($this->isMageCommunity()) {
                $this->createCommunityRewrite($store, $idPath, $requestPath, $targetPath);
            }
            else if ($this->isMageEnterprise()) {
                $this->createEnterpriseRewrite($store, $idPath, $requestPath, $targetPath);
            }
            return false;
        } catch (Exception $e) {
            Mage::log(
                "Create rewrite failed. store: {$store->getStoreId()}, {$requestPath} to {$targetPath}",
                Zend_Log::ERR,
                'sb.log'
            );
            return false;
        }
    }

    /**
     * @param $store
     * @param $idPath
     * @param $requestPath
     * @param $targetPath
     * @return bool
     */
    private function createCommunityRewrite($store, $idPath, $requestPath, $targetPath)
    {
        // check if rewrites already exist
        $existingRewrite = Mage::getModel('core/url_rewrite')->loadByIdPath($idPath);
        // if they don't
        if ($existingRewrite->getUrlRewriteId() == null) {
            Mage::getModel('core/url_rewrite')
                ->setIsSystem(0)
                ->setStoreId($store->getStoreId())
                ->setOptions('RP')
                ->setIdPath($idPath)
                ->setRequestPath($requestPath)
                ->setTargetPath($targetPath)
                ->save();
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param $store
     * @param $idPath
     * @param $requestPath
     * @param $targetPath
     * @return bool
     */
    private function createEnterpriseRewrite($store, $idPath, $requestPath, $targetPath)
    {
        $existingRewrite = Mage::getModel('enterprise_urlrewrite/redirect')->getCollection()
            ->addFieldToFilter('target_path')
            ->getFirstItem();
        if (!$existingRewrite->getId()) {
            Mage::getModel('enterprise_urlrewrite/redirect')
                ->setStoreId($store->getStoreId())
                ->setOptions('RP')
                ->setIdentifier($idPath)
                ->setRequestPath($requestPath)
                ->setTargetPath($targetPath)
                ->save();
            return true;
        } else {
            return false;
        }
    }

    /**
     * True if the version of Magento currently being run is Community Edition
     *
     * @return boolean
     */
    public function isMageCommunity()
    {
        return !$this->isMageEnterprise() && !$this->isMageProfessional();
    }

    /**
     * True if the version of Magento currently being run is Enterprise Edition
     *
     * @return boolean
     */
    public function isMageEnterprise()
    {
        return Mage::getConfig()->getModuleConfig('Enterprise_Enterprise')
            && Mage::getConfig()->getModuleConfig('Enterprise_AdminGws')
            && Mage::getConfig()->getModuleConfig('Enterprise_Checkout')
            && Mage::getConfig()->getModuleConfig('Enterprise_Customer');
    }
    /**
     * True if the version of Magento currently being run is Professional Edition
     *
     * @return boolean
     */
    public function isMageProfessional()
    {
        return Mage::getConfig()->getModuleConfig('Enterprise_Enterprise')
            && !Mage::getConfig()->getModuleConfig('Enterprise_AdminGws')
            && !Mage::getConfig()->getModuleConfig('Enterprise_Checkout')
            && !Mage::getConfig()->getModuleConfig('Enterprise_Customer');
    }

}
