<?php

class Springbot_Main_Model_Webhook
{
    private $apiToken;
    private $apiUrl;

    public function __construct()
    {
        $this->apiToken = Mage::getStoreConfig('springbot/main/api_token');
        $this->apiUrl = trim(Mage::getStoreConfig('springbot/main/api_url'), '/');
        if ($this->apiToken === null) {
            throw new Exception('API token not set, ensure the store has authed to Springbot', 500);
        }
    }

    /**
     * @param string $endpoint
     * @param int $id
     * @param array $storeIds
     */
    public function update($endpoint, $id, $storeIds)
    {
        $api = Mage::getModel('springbot_main/api');
        foreach ($this->getValidStoreIds($storeIds) as $storeId) {
            $springbotStoreId = $this->getSpringbotStoreId($storeId);
            $url = $this->getApiUrl("magento2/webhooks/v1/{$springbotStoreId}/{$endpoint}");
            $body = json_encode(array('id' => $id));
            $api->post($url, $body);
        }
    }

    /**
     * @param string $endpoint
     * @param int $id
     * @param array $storeIds
     */
    public function delete($endpoint, $id, $storeIds)
    {
        $api = Mage::getModel('springbot_main/api');
        foreach ($this->getValidStoreIds($storeIds) as $storeId) {
            $springbotStoreId = $this->getSpringbotStoreId($storeId);
            $url = $this->getApiUrl("magento2/webhooks/v1/{$springbotStoreId}/{$endpoint}");
            $body = json_encode(array('id' => $id, 'is_deleted' => true));
            $api->post($url, $body);
        }
    }

    /**
     * @param null|string $subpath
     * @return string
     */
    private function getApiUrl($subpath = null)
    {
        $url = $this->apiUrl;
        if ($subpath) {
            $url .= '/' . $subpath;
        }
        return $url;
    }

    /**
     * @param array $storeIds
     * @return array mixed
     */
    private function getValidStoreIds($storeIds)
    {
        $storeZeroAlias = Mage::getStoreConfig("springbot/main/store_zero");
        foreach ($storeIds as $index => $storeId) {
            if (!is_numeric($storeId)) {
                unset($storeIds[$index]);
            }
            if ($storeId == 0) {
                if ($storeZeroAlias) {
                    $storeIds[$index] = $storeZeroAlias;
                }
                else {
                    unset($storeIds[$index]);
                }
            }
        }
        return array_unique($storeIds);
    }

    /**
     * @param int $localStoreId
     * @return int
     * @throws Exception
     */
    private function getSpringbotStoreId($localStoreId)
    {
        $springbotStoreId = Mage::getStoreConfig("springbot/main/store_id_{$localStoreId}");
        if ($springbotStoreId === null) {
            throw new Exception("No Springbot store ID for local store {$localStoreId}", 500);
        }
        return $springbotStoreId;
    }

}
