<?php

class Springbot_Main_Observer_Entity_Inventory extends Springbot_Main_Observer_AbstractObserver
{
    const endpoint = 'inventories';

    /**
     * @param Varien_Event_Observer $observer
     */
    public function onCatalogInventorySave(Varien_Event_Observer $observer)
    {
        if ($item = $observer->getEvent()->getItem()) {
            $this->enqueueItem($item);
        }
    }

    /**
     * @param Varien_Event_Observer $observer
     */
    public function onQuoteSubmit(Varien_Event_Observer $observer)
    {
        if ($quote = $observer->getEvent()->getQuote()) {
            foreach ($quote->getAllItems() as $item) {
                $this->enqueueItem($item);
            }
        }
    }

    /**
     * @param Varien_Event_Observer $observer
     */
    public function onCancelOrderItem(Varien_Event_Observer $observer)
    {
        if ($item = $observer->getEvent()->getItem()) {
            $this->enqueueItem($item);
        }
    }

    /**
     * @param Varien_Event_Observer $observer
     */
    public function onCreditMemoSave(Varien_Event_Observer $observer)
    {
        if ($creditMemo = $observer->getEvent()->getCreditmemo()) {
            foreach ($creditMemo->getAllItems() as $creditMemoItem) {
                $productId = $creditMemoItem->getProductId();
                if ($product = Mage::getModel('catalog/product')->load($productId)) {
                    if ($item = Mage::getModel('cataloginventory/stock_item')->loadByProduct($product)) {
                        $this->enqueueItem($item, $product);
                    }
                }
            }
        }
    }

    /**
     * @param Mage_CatalogInventory_Model_Stock_Item $item
     * @param Mage_Catalog_Model_Product|null $product
     */
    private function enqueueItem($item, $product = null)
    {
        if ($product === null) {
            $product = $item->getProduct();
        }
        if ($product) {
            $storeIds = $product->getStoreIds();
            $this->queue->enqueue(self::webhook_class, 'update', array(self::endpoint, $item->getId(), $storeIds));
        }
    }

}