<?php

class Springbot_Main_Observer_Entity_Order extends Springbot_Main_Observer_AbstractObserver
{

    const endpoint = 'orders';
    const sb_redirect_cookie = 'springbot_redirect_queue';

    /**
     * @param Varien_Event_Observer $observer
     */
    public function onFrontendOrderSaveAfter($observer)
    {
        if ($order = $observer->getEvent()->getOrder()) {
            $storeIds = array($order->getStoreId());
            $this->queue->enqueue(self::webhook_class, 'update', array(self::endpoint, $order->getId(), $storeIds));
            if ($order->getCustomerIsGuest()) {
                $this->queue->enqueue(self::webhook_class, 'update', array('guests', ($order->getId() + Mage::getStoreConfig('springbot/main/guest_offset')), $storeIds));
            }
        }
    }

    /**
     * Fired after an order is placed, but not every time an order is saved. This is when we associate the redirect IDs
     * to the order for tracking purposes.
     *
     * @param Varien_Event_Observer $observer
     */
    public function onFrontendOrderPlaceAfter($observer)
    {
        if ($order = $observer->getEvent()->getOrder()) {
            if ($order->getId()) {
                $this->setRedirectOrderIds($order);
                $this->createTrackables($order->getId());
            }
        }
    }

    /**
     * @param Varien_Event_Observer $observer
     */
    public function onAdminOrderSaveAfter($observer)
    {
        if ($order = $observer->getEvent()->getOrder()) {
            $storeIds = array($order->getStoreId());
            $this->queue->enqueue(self::webhook_class, 'update', array(self::endpoint, $order->getId(), $storeIds));
            if ($order->getCustomerIsGuest()) {
                $this->queue->enqueue(self::webhook_class, 'update', array('guests', ($order->getId() + Mage::getStoreConfig('springbot/main/guest_offset')), $storeIds));
            }
        }
    }

    /**
     * @param $order
     */
    private function setRedirectOrderIds($order)
    {
        if ($redirectIdsStr = Mage::getModel('core/cookie')->get(self::sb_redirect_cookie)) {
            if (substr_count($redirectIdsStr, '%7') > 0) {
                $delimiter = '%7';
            }
            else {
                $delimiter = '|';
            }
            $redirectIds = explode($delimiter, $redirectIdsStr);
            foreach ($redirectIds as $redirectId) {
                Mage::getModel('springbot_main/redirect_quote')->create($order->getQuoteId(), $redirectId);
            }
        }
    }

    /**
     * @param $orderId
     */
    private function createTrackables($orderId)
    {
        $userAgent = Mage::helper('core/http')->getHttpUserAgent();
        return Mage::getModel('springbot_main/trackable_order')->create($orderId, 'user_agent', $userAgent);
    }

}