<?php

class Springbot_Main_ConfigController extends Mage_Core_Controller_Front_Action
{

    const basename = 'springbot/';

    /**
     * Return all configuration values in the springbot namespace
     */
    public function viewConfigAction()
    {
        $configValues = $this->recurseConfig('springbot', Mage::getStoreConfig('springbot'));
        $this->getResponse()->setBody(json_encode($configValues));
    }

    /**
     * Return the currently installed plugin version
     */
    public function getVersionAction()
    {
        $out = (array) Mage::getConfig()->getNode()->modules->Springbot_Main->version;
        $this->getResponse()->setBody(json_encode(array('version' => $out[0])));
    }

    /**
     * @param $path
     * @param $array
     * @return array
     */
    private function recurseConfig($path, $array)
    {
        $ret = array();
        foreach ($array as $key => $configValue) {
            if (is_array($configValue)) {
                $ret = array_merge($ret, $this->recurseConfig($path . '/' . $key, $configValue));
            } else {
                $ret[] = array(
                    'path'  => $path . '/' . $key,
                    'value' => $configValue
                );
            }
        }
        return $ret;
    }

    /**
     * Set a configuration value based on path and value
     *
     * @throws Exception
     */
    public function setConfigAction()
    {
        if ($decoded = json_decode($this->getRequest()->getRawBody(), true)) {

            if (!isset($decoded['path'])) {
                throw new Exception("Required key 'path' not set", 400);
            }
            if (substr($decoded['path'], 0, strlen(self::basename)) !== self::basename) {
                throw new Exception("Path must start with " . self::basename, 400);
            }
            if (!isset($decoded['value'])) {
                throw new Exception("Required key 'value' not set", 400);
            }

            Mage::getModel('core/config')->saveConfig($decoded['path'], $decoded['value'], 'default', 0);

            $this->getResponse()->setBody(json_encode(array(
                'successful' => true
            )));
        }
    }


}