<?php

class Springbot_Main_ProductChildrenController extends Mage_Core_Controller_Front_Action
{

    /**
     * Return basic information about the product and its children on the frontend
     */
    public function viewProductChildrenAction()
    {

        $this->getResponse()->setHeader('Content-type', 'application/json');
        try {

            $product = Mage::getModel('catalog/product')
                ->load($this->getRequest()->getParam('product_id'));

            if ( ! ($product instanceof Mage_Catalog_Model_Product)
                || ($product->getId() == 0)
            ) {
                throw new Exception('product not found');
            }

            $children = array();
            if ($product->isConfigurable()) {
                $childProducts = Mage::getModel(
                    'catalog/product_type_configurable'
                )
                    ->getUsedProducts(null, $product);

                $superAttributes = $product
                    ->getTypeInstance(true)
                    ->getConfigurableAttributesAsArray($product);

                foreach ($childProducts as $childProduct) {

                    $superAttributeValues   = array();
                    $superAttributeValues[] = $product->getName();
                    foreach ($superAttributes as $superAttribute) {
                        $superAttributeValues[] = $childProduct
                            ->getResource()
                            ->getAttribute($superAttribute['attribute_code'])
                            ->getFrontend()
                            ->getValue($childProduct);
                    }
                    $name       = implode(' / ', $superAttributeValues);
                    $children[] = array(
                        'name'       => $name,
                        'sku'        => $childProduct->getSku(),
                        'product_id' => $childProduct->getId()
                    );
                }
            } else {
                $children[] = array(
                    'name'       => $product->getName(),
                    'sku'        => $product->getSku(),
                    'product_id' => $product->getId()
                );
            }

            $this->getResponse()->setBody(
                json_encode(
                    array(
                        'successful' => true,
                        'name'       => $product->getName(),
                        'sku'        => $product->getSku(),
                        'product_id' => $product->getId(),
                        'children'   => $children
                    )
                )
            );

        } catch (Exception $e) {

            $this->getResponse()->setBody(
                json_encode(
                    array(
                        'successful' => false,
                        'message'    => $e->getMessage()
                    )
                )
            );

        }

    }

}
