<?php

class Springbot_Queue_Model_Job extends Mage_Core_Model_Abstract
{

    protected function _construct()
    {
        $this->_init('springbot_queue/job');
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function run()
    {
        $class = $this->getData('class');
        $method = $this->getData('method');
        $args = json_decode($this->getData('args'), true);

        if (!class_exists($class)) {
            throw new Exception("Class {$class} does not exist");
        }
        $object = new $class();

        if (!method_exists($object, $method)) {
            throw new Exception("Method {$method} does not exist in class {$class}");
        }

        return call_user_func_array(array($object, $method), $args);
    }

    /**
     * Method to run before a job has run
     */
    public function preRun()
    {
        $this->addData(array(
            'locked_by' => getmypid(),
            'locked_at' => time()
        ));
        $this->save();
    }

    /**
     * Method to run after a job has failed
     *
     * @param string $message
     */
    public function postRun($message)
    {
        $attempts = $this->getData('attempts');
        $this->addData(array(
            'attempts'    => $attempts + 1,
            'next_run_at' => time(),
            'error'       => $message,
            'locked_by'   => null,
            'locked_at'   => null
        ));
        $this->save();
    }

}