<?php

class Springbot_Router_Controller_Router extends Mage_Core_Controller_Varien_Router_Standard
{

    const endpoint_name = 'springbot';

    /**
     * Route requests for /springbot to the appropriate module, controller, and action
     *
     * @param Zend_Controller_Request_Http $request
     * @return bool
     */
    public function match(Zend_Controller_Request_Http $request)
    {

        try {
            // Verifies that Magento is installed, otherwise it redirects to install page
            if (!$this->_afterModuleMatch()) {
                return false;
            }

            // Parse the request path
            $path = trim($request->getPathInfo(), '/');
            $parts = explode('/', $path);

            // If the path doesn't start with our endpoint then ignore
            $endpoint = array_shift($parts);
            if ($endpoint !== self::endpoint_name) {
                return false;
            }

            // Prepare the response
            $response = $this->getFront()->getResponse();
            $response->setHeader('Content-type', 'application/json');
            $request->setDispatched(true);
            $auth = Mage::getModel('springbot_router/auth');
            $headerAuth = $request->getHeader('Authorization');
            if (!$headerAuth) {
                $headerAuth = $request->getHeader('Auth');
            }

            // Load the routes from the XML configuration and traverse it
            $routes = Mage::getConfig()->getNode('default/springbot_routes')->asArray();

            $attributes = array();

            while ($part = array_shift($parts)) {
                if (isset($routes[$part])) {
                    // Verify there is a matching path in the XML routes definitions
                    $routes = $routes[$part];
                    $attributes = $this->extractAttributes($routes, $attributes);
                }
                else if (isset($routes['_'])) {
                    // Underscores represent wildcards in the path, and their values are passed to the request object
                    $routes = $routes['_'];
                    $attributes = $this->extractAttributes($routes, $attributes);
                    $request->setParam($attributes['name'], $part);
                }
                else {
                    throw new Exception("Route not found", 404);
                }
            }

            if (!$auth->validate($headerAuth)) {
                if (isset($attributes['sbauth'])) {
                    if ($attributes['sbauth'] !== 'false') {
                        throw new Exception('This endpoint requires authentication.', 401);
                    }   
                } else {
                    throw new Exception('This endpoint requires authentication.', 401);
                }
            }

            // Verify all the pieces we need were set in the XML
            $method = strtolower($request->getMethod());
            $this->verifyKeysSet(array('module', 'controller', $method), $attributes);

            // Instantiate the controller
            $controllerClass = $attributes['module'] . '_' . ucwords($attributes['controller']) . 'Controller';
            $this->_validateControllerClassName($attributes['module'], $attributes['controller']);
            $controllerInstance = Mage::getControllerInstance($controllerClass, $request, $response);

            // Set values after being routed successfully
            $request->setRouteName($this->getRouteByFrontName($endpoint));
            $request->setModuleName($endpoint);
            $request->setControllerName($attributes['controller']);
            $request->setActionName($attributes[$method]);
            $request->setControllerModule($attributes['module']);

            // Dispatch the action to the controller
            $controllerInstance->dispatch($attributes[$method]);

        } catch (Exception $e) {
            //$response->setHttpResponseCode($e->getCode());
            $code = $e->getCode();
            if ($code >= 200) {
                $response->setHeader('HTTP/1.0', $code, true);
            }
            $response->setBody(Mage::helper('core')->jsonEncode(array(
                'error'       => $e->getCode(),
                'message'     => $e->getMessage(),
                'stack_trace' => $e->getTraceAsString()
            )));
        }

        return true;
    }

    /**
     * @param array $array
     * @param array $existing
     * @return array
     */
    private function extractAttributes($array, $existing)
    {
        if (is_array($array) && isset($array['@'])) {
            return array_merge($existing, $array['@']);
        }
        return $existing;
    }

    /**
     * @param array $keys
     * @param $array
     * @throws Exception
     */
    private function verifyKeysSet(array $keys, $array)
    {
        foreach ($keys as $key) {
            if (!isset($array[$key])) {
                throw new Exception("Action not set for {$key} method");
            }
        }
    }

}
