<?php

class Springbot_Router_Model_Auth extends Mage_Core_Model_Abstract
{

    const access_token_path = 'springbot/router/access_token';

    /**
     * Validate the request for the supplied access token
     *
     * @param $accessToken
     * @return bool
     */
    public function validate($accessToken)
    {
        $correctAccessToken = $this->getAccessToken();
        if ($accessToken === "Bearer {$correctAccessToken}") {
            return true;
        }
        return false;
    }

    /**
     * Retrieve the access token if it exists, otherwise generate it. Return the result.
     * @return string
     */
    public function getAccessToken()
    {
        $accessToken = Mage::getStoreConfig(self::access_token_path);
        if (!$accessToken) {
            $accessToken = $this->generateRandomAccessToken();
            Mage::getModel('core/config')->saveConfig(self::access_token_path, $accessToken, 'default', 0);
        }
        return $accessToken;
    }

    /**
     * Generate a random access token of a given length
     *
     * @param int $length
     * @return string
     */
    private function generateRandomAccessToken($length = 32)
    {
        return substr(
            str_shuffle(
                str_repeat(
                    $x = '0123456789abcdefghijklmnopqrstuvwxyz', ceil($length / strlen($x))
                )
            ),1, $length
        );
    }

}
